<?php

/**
 * @file
 * Hooks provided by the Remote Entity API.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Provide an entity type via the remote entity API.
 *
 * This is a placeholder for describing further keys for hook_entity_info(),
 * which are introduced by the remote entity API for providing a new entity type
 * in sync with a remote entity.
 *
 * The following core or Entity API keys are used by the remote entity API:
 *
 * - 'controller class': This should be 'RemoteEntityAPIDefaultController' or
 *    a subclass.
 * - 'metadata controller class': This should be
 *   'RemoteEntityAPIDefaultMetadataController' or a subclass.
 *
 * The following core or Entity API keys are extended by the remote entity API:
 *
 * - 'admin ui': If the entity uses RemoteEntityEntityUIController as its
 *   controller, then this may have the following additional keys:
 *    - 'form buttons': An array specifying which buttons should be shown on the
 *      form, in the format 'button_name' => TRUE. Possible button names are:
 *      - 'fetch': Show a button to fetch all remote entities. (Only use this
 *        when you know the number of remote entities is relatively small!)
 *
 * The remote entity API supports the following keys:
 *
 * - 'remote base table': The remote base table or entity type. This is used
 *   in remote queries.
 * - 'remote query defaults callback': (Optional) A function with signature
 *   callback_remote_entity_query_defaults(), which is called on all new
 *   RemoteEntityQuery objects. This may be used to set default conditions or
 *   relationships on queries.
 * - 'remote entity keys': An array of properties on the Drupal entity that map
 *   to properties on the remote data.
 *    - 'remote id': The remote entity property which gives its remote ID.
 *    - 'label': The remote entity property which gives its label, if
 *      remote_entity_entity_label() is being used as the label callback.
 * - 'property map': An array of mappings from local property names to remote
 *   entity properties. Local names will be made available via the Entity
 *   metadata API. For instance, the key-value pair 'foo' => 'bar' will mean
 *   that $wrapper->foo->value() will return the value of $remote_entity->bar.
 * - 'expiry': (optional) Allows local copies of entities to expire, after
 *   which time they are refreshed. If this is not set, local entities never
 *   expire. This should be an array with the following properties:
 *    - 'expiry time': A value in seconds that determines how long a local copy
 *      of a remote entity is considered fresh. After this time, loading the
 *      local copy will result in a remote retrieval of the entity.
 *    - 'purge': A boolean indicating whether or not to delete expired local
 *      entities on cron. Warning: purging local entities will mean that when
 *      they are next retrieved, their local entity ID will change, thus
 *      breaking any entityreference field values and fields on the entities
 *      themselves.
 *
 * @see hook_entity_info()
 * @see entity_crud_hook_entity_info()
 */
function remote_entity_hook_entity_info() {

}

/**
 * Declare information about remote tables for RemoteEntityQuery.
 *
 * WARNING: This system is currently under active development, and may be
 * further changed in the future.
 */
function hook_remote_entity_query_table_info() {
  $data = array(
    // Table name.
    'mytable' => array(
      // Data about fields. Not all RemoteEntityQuery implementations will
      // necessarily need this, or need this for all contexts. Eg, these may
      // only be needed when used in conditions.
      'fields' => array(
        // The default type is 'string'; only define ones that differ.
        'typecode' => array(
          'type' => 'int',
        ),
        'statecode' => array(
          'type' => 'int',
        ),
      ),
      // Joins from this table to others.
      'relationships' => array(
        // Relationship name. Arbitrary, used in adding to the query.
        'myrelationship' => array(
          // The LHS of the join, starting from this table.
          'base' => array(
            // The field on this table.
            'field' => 'address1_addressid',
          ),
          // The RHS of the join.
          'join' => array(
            'table' => 'customeraddress',
            'field' => 'customeraddressid',
          ),
        ),
      ),
    ),
  );

  return $data;
}

/**
 * @addtogroup callbacks
 * @{
 */

/**
 * Allows defaults to be set on all RemoteEntityQueries for an entity type.
 *
 * @param RemoteEntityQuery $query
 *  A RemoteEntityQuery, with the base already set by the entity controller.
 *  This is passed by reference and should be altered directly.
 *  The entity info for the base entity can be found in $query->entity_info.
 *
 * @see remote_entity_hook_entity_info()
 * @see RemoteEntityAPIDefaultController::getRemoteEntityQuery()
 */
function callback_remote_entity_query_defaults($query) {
  // We only work with remote entities of property foo equal to bar.
  $query->fieldRemoteCondition('foo', 'bar');
}
